@@ -1,17 +1,21 @@
 #!/usr/bin/env python
+from __future__ import print_function
+
 import argparse
 from collections import Counter
 from datetime import datetime
 import logging
-import os
-import re
+import re, sys
+import os, pycurl, tarfile, zipfile, gzip, shutil
+from pkg_resources import resource_filename
+
 
 from sistr.version import __version__
 from sistr.src.blast_wrapper import BlastRunner
 from sistr.src.cgmlst import run_cgmlst
 from sistr.src.logger import init_console_logger
 from sistr.src.qc import qc
-from sistr.src.serovar_prediction import SerovarPredictor, overall_serovar_call, serovar_table
+from sistr.src.serovar_prediction import SerovarPredictor, overall_serovar_call, serovar_table, SISTR_DB_URL, SISTR_DATA_DIR
 
 
 def init_parser():
@@ -41,6 +45,7 @@ PLoS ONE 11(1): e0147101. doi: 10.1371/journal.pone.0147101
     parser.add_argument('-i',
                         '--input-fasta-genome-name',
                         nargs=2,
+                        metavar=('fasta_path', 'genome_name'),
                         action='append',
                         help='fasta file path to genome name pair')
     parser.add_argument('-f',
@@ -50,6 +55,11 @@ PLoS ONE 11(1): e0147101. doi: 10.1371/journal.pone.0147101
     parser.add_argument('-o',
                         '--output-prediction',
                         help='SISTR serovar prediction output path')
+    parser.add_argument('-M',
+                        '--more-results',
+                        action='count',
+                        default=0,
+                        help='Output more detailed results (-M) and all antigen search blastn results (-MM)')
     parser.add_argument('-p',
                         '--cgmlst-profiles',
                         help='Output CSV file destination for cgMLST allelic profiles')
@@ -86,7 +96,7 @@ PLoS ONE 11(1): e0147101. doi: 10.1371/journal.pone.0147101
     parser.add_argument('-v',
                         '--verbose',
                         action='count',
-                        default=2,
+                        default=0,
                         help='Logging verbosity level (-v == show warnings; -vvv == show debug info)')
     parser.add_argument('-V', '--version', action='version', version='%(prog)s {}'.format(__version__))
     return parser
@@ -97,6 +107,17 @@ def run_mash(input_fasta):
 
     mash_out = mash_dist_trusted(input_fasta)
     df_mash = mash_output_to_pandas_df(mash_out)
+    if df_mash.empty:
+        logging.error('Could not perform Mash subspeciation!')
+        mash_result_dict = {
+            'mash_genome': '',
+            'mash_serovar': '',
+            'mash_distance': 1.0,
+            'mash_match': 0,
+            'mash_subspecies': '',
+            'mash_top_5': {},
+        }
+        return mash_result_dict
     df_mash_top_5 = df_mash[['ref', 'dist', 'n_match', 'serovar']].head(n=5)
     logging.debug('Mash top 5 results:\n{}\n'.format(df_mash_top_5))
     mash_spp_tuple = mash_subspeciation(df_mash)
@@ -138,27 +159,82 @@ def merge_cgmlst_prediction(serovar_prediction, cgmlst_prediction):
     serovar_prediction.cgmlst_distance = cgmlst_prediction['distance']
     serovar_prediction.cgmlst_genome_match = cgmlst_prediction['genome_match']
     serovar_prediction.serovar_cgmlst = cgmlst_prediction['serovar']
+    if 'found_loci' in cgmlst_prediction:
+        serovar_prediction.cgmlst_found_loci = cgmlst_prediction['found_loci']
+    else:
+        serovar_prediction.cgmlst_found_loci = 0
     serovar_prediction.cgmlst_matching_alleles = cgmlst_prediction['matching_alleles']
     serovar_prediction.cgmlst_subspecies = cgmlst_prediction['subspecies']
     serovar_prediction.cgmlst_ST = cgmlst_prediction['cgmlst330_ST']
     return serovar_prediction
 
 
+
 def infer_o_antigen(prediction):
     df_serovar = serovar_table()
-    predicted_serovars = prediction.serovar.split('|') if '|' in prediction.serovar else [prediction.serovar]
-    series_o_antigens = df_serovar.O_antigen[df_serovar.Serovar.isin(predicted_serovars)]
-    if series_o_antigens.size == 0:
-        sg = prediction.serogroup
-        if sg is None or sg == '' or sg == '-':
+    if '|' in prediction.serovar:
+        prediction.o_antigen = '-'
+    else:
+        predicted_serovars = [prediction.serovar]
+        series_o_antigens = df_serovar.O_antigen[df_serovar.Serovar.isin(predicted_serovars)]
+        if series_o_antigens.size == 0:
             prediction.o_antigen = '-'
         else:
-            series_o_antigens = df_serovar.O_antigen[df_serovar.Serogroup == sg]
             counter_o_antigens = Counter(series_o_antigens)
             prediction.o_antigen = counter_o_antigens.most_common(1)[0][0]
+
+def download_to_file(url,file):
+    with open(file, 'wb') as f:
+        c = pycurl.Curl()
+        # Redirects to https://www.python.org/.
+        c.setopt(c.URL, url)
+        # Follow redirect.
+        c.setopt(c.FOLLOWLOCATION, True)
+        c.setopt(c.WRITEDATA, f)
+        c.perform()
+        c.close()
+
+def extract(fname,outdir):
+    if (fname.endswith("tar.gz")):
+        tar = tarfile.open(fname, "r:gz")
+        tar.extractall(outdir)
+        tar.close()
+    elif (fname.endswith("tar")):
+        tar = tarfile.open(fname, "r:")
+        tar.extractall(outdir)
+        tar.close()
+    elif(fname.endswith("zip")):
+        zip_ref = zipfile.ZipFile(fname, 'r')
+        zip_ref.extractall(outdir)
+        zip_ref.close()
+    elif(fname.endswith("gz")):
+        outfile = os.path.join(outdir,fname.replace('.gz',''))
+        with gzip.open(fname, 'rb') as f_in:
+            with open(outfile, 'wb') as f_out:
+                shutil.copyfileobj(f_in, f_out)
+            f_in.close()
+            f_out.close()
+            os.remove(fname)
+
+def setup_sistr_dbs():
+    tmp_file = resource_filename('sistr', 'data.tar.gz')
+    logging.info("Downloading needed SISTR databases from: {}".format(SISTR_DB_URL))
+    download_to_file(SISTR_DB_URL, tmp_file)
+    if os.path.isdir(resource_filename('sistr', 'data/')):
+        shutil.rmtree(resource_filename('sistr', 'data/'))
+        os.mkdir(resource_filename('sistr', 'data/'))
+    if (not os.path.isfile(tmp_file)):
+        logging.error('Downloading databases failed, please check your internet connection and retry')
+        sys.exit(-1)
     else:
-        counter_o_antigens = Counter(series_o_antigens)
-        prediction.o_antigen = counter_o_antigens.most_common(1)[0][0]
+        logging.info('Downloading databases successful')
+        f = open(resource_filename('sistr', 'dbstatus.txt'),'w')
+        f.write("DB downloaded on : {} from {}".format(datetime.today().strftime('%Y-%m-%d'),SISTR_DB_URL))
+        f.close()
+
+    extract(tmp_file, resource_filename('sistr', ''))
+    os.remove(tmp_file)
+
 
 
 def sistr_predict(input_fasta, genome_name, tmp_dir, keep_tmp, args):
@@ -168,7 +244,8 @@ def sistr_predict(input_fasta, genome_name, tmp_dir, keep_tmp, args):
         if genome_name is None or genome_name == '':
             genome_name = genome_name_from_fasta_path(input_fasta)
         dtnow = datetime.now()
-        genome_tmp_dir = os.path.join(tmp_dir, dtnow.strftime("%Y%m%d%H%M%S") + '-' + 'SISTR' + '-' + genome_name)
+        genome_name_no_spaces = re.sub(r'\W', '_', genome_name)
+        genome_tmp_dir = os.path.join(tmp_dir, dtnow.strftime("%Y%m%d%H%M%S") + '-' + 'SISTR' + '-' + genome_name_no_spaces)
         blast_runner = BlastRunner(input_fasta, genome_tmp_dir)
         logging.info('Initializing temporary analysis directory "%s" and preparing for BLAST searching.', genome_tmp_dir)
         blast_runner.prep_blast()
@@ -187,6 +264,7 @@ def sistr_predict(input_fasta, genome_name, tmp_dir, keep_tmp, args):
 
         serovar_predictor = SerovarPredictor(blast_runner, spp)
         serovar_predictor.predict_serovar_from_antigen_blast()
+
         prediction = serovar_predictor.get_serovar_prediction()
         prediction.genome = genome_name
         prediction.fasta_filepath = os.path.abspath(input_fasta)
@@ -245,7 +323,6 @@ def genome_name_from_fasta_path(fasta_path):
     return re.sub(r'(\.fa$)|(\.fas$)|(\.fasta$)|(\.fna$)|(\.\w{1,}$)', '', filename)
 
 
-
 def write_cgmlst_profiles(fastas, cgmlst_results, output_path):
     genome_marker_cgmlst_result = {}
     for genome, res in zip(fastas, cgmlst_results):
@@ -282,13 +359,19 @@ def write_novel_alleles(cgmlst_results, output_path):
 
 
 def main():
+
     parser = init_parser()
     args = parser.parse_args()
     init_console_logger(args.verbose)
+    logging.info('Running sistr_cmd {}'.format(__version__))
+    if not os.path.isfile(resource_filename('sistr', 'dbstatus.txt')):
+        setup_sistr_dbs()
     input_fastas = args.fastas
     paths_names = args.input_fasta_genome_name
-    if len(input_fastas) == 0 and paths_names and len(paths_names) == 0:
-        raise Exception('No FASTA files specified!')
+    if len(input_fastas) == 0 and (paths_names is None or len(paths_names) == 0):
+        logging.error('No FASTA files specified!')
+        parser.print_help()
+        sys.exit(-1)
     if paths_names is None:
         genome_names = [genome_name_from_fasta_path(x) for x in input_fastas]
     else:
@@ -300,22 +383,24 @@ def main():
             input_fastas = [x for x,y in paths_names] + tmp
             genome_names = [y for x,y in paths_names] + [genome_name_from_fasta_path(x) for x in tmp]
         else:
-            raise Exception('Unhandled fasta input args: input_fastas="{}" | input_fasta_genome_name="{}"'.format(
+            logging.error('Unhandled fasta input args: input_fastas="{}" | input_fasta_genome_name="{}"'.format(
                 input_fastas,
                 paths_names))
+            parser.print_help()
+            sys.exit(-1)
 
     tmp_dir = args.tmp_dir
     keep_tmp = args.keep_tmp
     output_format = args.output_format
     output_path = args.output_prediction
 
-    from multiprocessing import Pool
-    n_threads = args.threads
 
+    n_threads = args.threads
     if n_threads == 1:
         logging.info('Serial single threaded run mode on %s genomes', len(input_fastas))
         outputs = [sistr_predict(input_fasta, genome_name, tmp_dir, keep_tmp, args) for input_fasta, genome_name in zip(input_fastas, genome_names)]
     else:
+        from multiprocessing import Pool
         logging.info('Initializing thread pool with %s threads', n_threads)
         pool = Pool(processes=n_threads)
         logging.info('Running SISTR analysis asynchronously on %s genomes', len(input_fastas))
@@ -325,23 +410,40 @@ def main():
         outputs = [x.get() for x in res]
 
     prediction_outputs = [x for x,y in outputs]
+
     cgmlst_results = [y for x,y in outputs]
 
     if output_path:
         from sistr.src.writers import write
-        write(output_path, output_format, prediction_outputs)
+        logging.info('Writing results with %s verbosity',
+                     args.more_results)
+        write(output_path, output_format, prediction_outputs, more_results=args.more_results)
     else:
-        logging.warning('No prediction results output file written!')
+        import json
+        from sistr.src.writers import to_dict
+        logging.warning('No prediction results output file written! Writing results summary to stdout as JSON')
+        exclude_keys_in_output = {'blast_results', 'sseq'}
+        if args.more_results >= 2:
+            exclude_keys_in_output.remove('blast_results')
+            exclude_keys_in_output.remove('sseq')
+        elif args.more_results == 1:
+            exclude_keys_in_output.remove('sseq')
+        outs = [to_dict(x, 0, exclude_keys=exclude_keys_in_output) for x in prediction_outputs]
+        print(json.dumps(outs))
     if args.cgmlst_profiles:
-        write_cgmlst_profiles(input_fastas, cgmlst_results, args.cgmlst_profiles)
+        write_cgmlst_profiles(genome_names, cgmlst_results, args.cgmlst_profiles)
         logging.info('cgMLST allelic profiles written to %s', args.cgmlst_profiles)
     if args.alleles_output:
-        write_cgmlst_results_json(input_fastas, cgmlst_results, args.alleles_output)
+        write_cgmlst_results_json(genome_names, cgmlst_results, args.alleles_output)
         logging.info('JSON of allele data written to %s for %s cgMLST allele results', args.alleles_output, len(cgmlst_results))
     if args.novel_alleles:
         count = write_novel_alleles(cgmlst_results, args.novel_alleles)
         logging.info('Wrote %s alleles to %s', count, args.novel_alleles)
 
 
+
+
 if __name__ == '__main__':
     main()
+
+
